{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit DDODocumentTests;

{$I DDObjectsTests.inc}

interface

uses
  SysUtils, Classes, TestFramework,
  DDOTestExtensions, DDODocument;

type
  TDDODocumentTests = class(TDDOBaseTest)
  private
    procedure CheckDocument(Document: TDDODocument);
    function LoadFile(FileName: String): String;
  published
    procedure TestParseXML;
    procedure TestGetXML;
    procedure TestCloneDocument;
    procedure PerformanceTest; override;
    procedure TestCloneNodes;
    procedure TestDocumentHandler;
  end;

implementation

const
  DOCFILE1 = 'resources\testdoc.xml';
  DOCFILE2 = 'resources\testdoc2.xml';

type
  TTestDocumentHandler = class(TInterfacedObject, IDDODocumentHandler)
  private
    FDocumentStartedHit,
    FDocumentCompletedHit: Boolean;
    FNodeCount: Integer;
  protected
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  public
    procedure DocumentStarted;
    procedure NodeCompleted(Node: TDDONode; var Continue: Boolean);
    function DocumentFailed: Boolean;
    procedure DocumentCompleted;
    procedure CheckHit;
  end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.TestCloneDocument;
var
  lDoc,
  lClone: TDDODocument;
begin
  lClone := nil;
  lDoc := TDDODocument.Create;
  try
    lDoc.LoadFromFile(DOCFILE1);
    lClone := lDoc.Clone;
    CheckEqualsString(lDoc.GetXML, lClone.GetXML);
  finally
    lClone.Free;
    lDoc.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.TestGetXML;
var
  lDoc: TDDODocument;
  lXML: String;
begin
  lXML := LoadFile(DOCFILE1);
  lDoc := TDDODocument.Create;
  try
    lDoc.LoadFromFile(DOCFILE1);
    CheckEqualsString(lXML, lDoc.GetXML);
  finally
    lDoc.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

function TDDODocumentTests.LoadFile(FileName: String): String;
var
  lStrings: TStrings;
  i: Integer;
begin
  lStrings := TStringList.Create;
  try
    lStrings.LoadFromFile(FileName);
    for i := 0 to lStrings.Count -1 do
      lStrings[i] := Trim(lStrings[i]);
    Result := lStrings.Text;
    Result := StringReplace(Result, #13, EmptyStr, [rfReplaceAll]);
    Result := StringReplace(Result, #10, EmptyStr, [rfReplaceAll]);
  finally
    lStrings.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.CheckDocument(Document: TDDODocument);
var
  i: Integer;
  lNode: TDDONode;
begin
  CheckEqualsString('root', Document.Root.NodeName);
  CheckEquals(2, Document.Root.ChildCount);
  for i := 0 to 1 do
    CheckEqualsString('node', Document.Root.Node[i].NodeName);
  CheckEquals(3, Document.Root.Node[0].ChildCount);
  for i := 0 to 2 do
  begin
    lNode := Document.Root.Node[0].Node[i];
    CheckEqualsString('subnode', lNode.NodeName);
    CheckEquals(1, lNode.Attributes.Count);
    CheckTrue(lNode.HasAttribute('name'));
    CheckEqualsString('node' + IntToStr(i + 1),
      lNode.Attributes['name']);
  end;
  CheckEquals(1, lNode.ChildCount);
  lNode := lNode.Node[0];
  CheckEqualsString('text', lNode.NodeName);
  CheckEqualsString('some text', lNode.NodeValue);
  lNode := Document.Root.Node[1];
  CheckEqualsString('node', lNode.NodeName);
  CheckEquals(1, lNode.ChildCount);
  lNode := lNode.Node[0];
  CheckEqualsString('subnode', lNode.NodeName);
  CheckTrue(lNode.HasAttribute('name'));
  CheckEqualsString('node4', lNode.Attributes['name']);
  CheckEquals(1, lNode.ChildCount);
  lNode := lNode.Node[0];
  CheckEqualsString('text', lNode.NodeName);
  CheckEqualsString('some >&<''-"-'' other text', lNode.NodeValue);
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.TestParseXML;
var
  lDoc: TDDODocument;
begin
  lDoc := TDDODocument.Create;
  try
    lDoc.LoadFromFile(DOCFILE1);
    CheckDocument(lDoc);
    lDoc.LoadFromFile(DOCFILE2);
    CheckDocument(lDoc);
  finally
    lDoc.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.PerformanceTest;
var
  i: integer;
begin
  for i := 1 to 500 do
    TestCloneDocument;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDODocumentTests.TestCloneNodes;
var
  lDoc,
  lClone: TDDODocument;
  i: integer;
begin
  lClone := nil;
  lDoc := TDDODocument.Create;
  try
    lDoc.LoadFromFile(DOCFILE1);
    lClone := TDDODocument.Create(lDoc.Root.NodeName);
    for i := 0 to lDoc.Root.ChildCount -1 do
      lClone.Root.Add(lDoc.Root.Node[i]);
    CheckEqualsString(lDoc.GetXML, lClone.GetXML);
  finally
    lClone.Free;
    lDoc.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

procedure TDDODocumentTests.TestDocumentHandler;
var
  lHandler: TTestDocumentHandler;
  lXML: String;
begin
  lXML := LoadFile(DOCFILE1);
  lHandler := TTestDocumentHandler.Create;
  try
    TDDODocument.Parse(lXML, lHandler);
    lHandler.CheckHit;
  finally
    lHandler.Free;
  end;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

procedure TTestDocumentHandler.DocumentCompleted;
begin
  FDocumentCompletedHit := True;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

procedure TTestDocumentHandler.DocumentStarted;
begin
  FDocumentStartedHit := True;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

function TTestDocumentHandler.DocumentFailed: Boolean;
begin
  Result := False;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

procedure TTestDocumentHandler.NodeCompleted(Node: TDDONode;
  var Continue: Boolean);
  procedure nCheckSubNodes(ACount: Integer; AName: String);
  var
    i: Integer;
  begin
    Assert(Node.ChildCount = ACount);
    for i := 0 to ACount -1 do
    begin
      Assert(Node.Node[i].NodeName = AName);
      Assert(Node.Node[i].Attributes.Value['name'] = 'node' +
        IntToStr((FNodeCount -1) * 3 + i +1));
    end;
  end;
begin
  Inc(FNodeCount);
  Assert(Node.NodeName = 'node');
  case FNodeCount of
    1: nCheckSubNodes(3, 'subnode');
    2: nCheckSubNodes(1, 'subnode');
  end;
end;

//----------------------------------------------------------------------------
//2006-12-14 00:09 N,SM:

procedure TTestDocumentHandler.CheckHit;
begin
  Assert(FDocumentStartedHit);
  Assert(FDocumentCompletedHit);
  Assert(FNodeCount = 2);
end;

//----------------------------------------------------------------------------
//2007-01-25 22:30 N,SM:

function TTestDocumentHandler._AddRef: Integer;
begin
  Result := -1;
end;

//----------------------------------------------------------------------------
//2007-01-25 22:30 N,SM:

function TTestDocumentHandler._Release: Integer;
begin
  Result := -1;
end;

initialization
  TestFramework.RegisterTest('DDOCommonTests',
    TDDODocumentTests.Suite);

end.