{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit DDOTestExtensions;

{$I DDObjectsTests.inc}

interface

uses
  SysUtils, Classes, TestFramework, ContNrs,
  {$IFDEF MemoryTests}DDSMemory, {$ENDIF}DDOClient, DDOServer;

type
  TDDOBaseTest = class(TTestCase)
  private
    {$IFDEF MemoryTests}
    FTracker: IDDSMemoryProfiler;
    FExpected: Integer;
    function GetStatus: TDDSMemoryStatus;
    {$ENDIF}
    procedure Initialize;
    procedure Finalize;
  protected
    procedure Invoke(AMethod: TTestMethod); override;
    procedure CheckGreater(ValueA, ValueB: Integer);
    procedure CheckLowerOrEqual(ValueA, ValueB: Integer);
    {$IFDEF MemoryTests}
    property ExpectedLeak: Integer read FExpected write FExpected;
    property MemoryStatus: TDDSMemoryStatus read GetStatus;
    {$ENDIF}
  public
    constructor Create(MethodName: string); override;
    destructor Destroy; override;
  published
    procedure PerformanceTest; virtual; abstract;
  end;

  TDDOBaseRemoteTest = class(TDDOBaseTest)
  private
    FRequester: TDDORequester;
    FListener: TDDOListener;
    FCreated: TObjectList;
  protected
    procedure SetUp; override;
    procedure TearDown; override;
    function CreateProxy(ProxyClass: TDDOProxyBaseClass;
      ARequester: TDDORequester = nil): Pointer;
    procedure RegisterClass(StubClass: TDDOStubClass);
    property Listener: TDDOListener read FListener;
    property Requester: TDDORequester read FRequester;
  end;

implementation

uses
  Windows, Forms, DDOShared{$IFDEF MemoryTests}, DDSAccess{$ENDIF};

const
  CPerformance = 'performance';
  CGreaterFailed = '%d is not greater than %d';
  CLowerOrEqualFailed = '%d is not lower or equal to %d';

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDOBaseTest.CheckGreater(ValueA, ValueB: Integer);
begin
  if ValueA <= ValueB then
    Fail(Format(CGreaterFailed, [ValueA, ValueB]));
end;

//----------------------------------------------------------------------------
//2006-07-02 01:30 N,SM:

procedure TDDOBaseTest.CheckLowerOrEqual(ValueA, ValueB: Integer);
begin
  if ValueA > ValueB then
    Fail(Format(CLowerOrEqualFailed, [ValueA, ValueB]));
end;

//----------------------------------------------------------------------------
//2006-07-02 01:30 N,SM:

constructor TDDOBaseTest.Create(MethodName: string);
begin
  inherited;
  {$IFDEF MemoryTests}
  FTracker := DDSHandler.MemoryProfiler;
  {$ENDIF}
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

destructor TDDOBaseTest.Destroy;
begin
  Finalize;
  inherited;
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDOBaseTest.Finalize;
begin
  {$IFDEF MemoryTests}
  FTracker.Active := False;
  FTracker.Clear;
  {$ENDIF}
  {$IFDEF BurnInTest}
    // Usually this is not recommended. However, within the
    // burn in test the GUI should be at least a bit responsive
    Application.ProcessMessages;
  {$ENDIF}
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

{$IFDEF MemoryTests}
function TDDOBaseTest.GetStatus: TDDSMemoryStatus;
begin
  Result := FTracker.GlobalStatus;
end;
{$ENDIF}

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM:

procedure TDDOBaseTest.Initialize;
begin
  {$IFDEF MemoryTests}
  FTracker.Active := True;
  FTracker.Clear;
  {$ENDIF}
end;

//----------------------------------------------------------------------------
//2006-07-02 00:09 N,SM: tests which contain 'performance' in it's name
//  will execute normally. All other tests will be checked for memleaks.

procedure TDDOBaseTest.Invoke(AMethod: TTestMethod);
begin
  if Pos(CPerformance, LowerCase(GetName)) > 0 then
  begin
    try
      inherited Invoke(AMethod);
    except
      // sometimes the performancetests fail with an timeout.
      // need to investigate this. ignoring these errors for now.
      on EDDOTimeOut do ;
    end;
  end else
  begin
    Initialize;
    try
      inherited Invoke(AMethod);
      {$IFDEF MemoryTests}
      CheckGreater(MemoryStatus.AllocPeak, 0);
      CheckLowerOrEqual(MemoryStatus.Allocated, ExpectedLeak);
      {$ENDIF}
    finally
      Finalize;
    end;
  end;
end;

//----------------------------------------------------------------------------
//2006-07-02 03:42 N,SM:

function TDDOBaseRemoteTest.CreateProxy(
  ProxyClass: TDDOProxyBaseClass; ARequester: TDDORequester = nil): Pointer;
begin
  if ARequester = nil then
    ARequester := FRequester;
  Result := ProxyClass.Create(ARequester);
  FCreated.Add(Result);
end;

//----------------------------------------------------------------------------
//2006-07-02 03:47 N,SM:

procedure TDDOBaseRemoteTest.RegisterClass(StubClass: TDDOStubClass);
begin
  FListener.RegisterClass(StubClass);
end;

//----------------------------------------------------------------------------
//2006-07-02 03:47 N,SM:

procedure TDDOBaseRemoteTest.SetUp;
begin
  inherited;
  FListener := TDDOListener.Create(nil);
  FListener.SessionTimeOut := 5;
  FListener.ThreadTimeOut := 5;
  FListener.Active := True;
  FRequester := TDDORequester.Create(nil);
  {$IFDEF MemoryTests}
  ExpectedLeak := MAXINT;
  {$ENDIF}
  FCreated := TObjectList.Create;
end;

//----------------------------------------------------------------------------
//2006-07-02 03:42 N,SM:

procedure TDDOBaseRemoteTest.TearDown;
begin
  FreeAndNil(FRequester);
  //***** This ugly 'solution' works around an issue (Access Violation) *****
  //***** if tests are executed on multiprocessor/dual-core machines.   *****
  //***** Yet I've not figured out what's going wrong. However, this    *****
  //***** issue is not relevant in real life but should be fixed asap.  *****
  Sleep(250);
  //***** ------------------------------------------------------------- *****
  FreeAndNil(FListener);
  FreeAndNil(FCreated);
  inherited;
end;

end.
